@@ -3,10 +3,7 @@ from ._api import delete, get, head, options, patch, post, put, request, stream
 from ._auth import Auth, BasicAuth, DigestAuth
 from ._client import AsyncClient, Client
 from ._config import PoolLimits, Proxy, Timeout
-from ._dispatch.asgi import ASGIDispatch
-from ._dispatch.wsgi import WSGIDispatch
 from ._exceptions import (
-    ConnectionClosed,
     ConnectTimeout,
     CookieConflict,
     DecodingError,
@@ -23,12 +20,14 @@ from ._exceptions import (
     ResponseClosed,
     ResponseNotRead,
     StreamConsumed,
-    TimeoutException,
     TooManyRedirects,
     WriteTimeout,
 )
 from ._models import URL, Cookies, Headers, QueryParams, Request, Response
 from ._status_codes import StatusCode, codes
+from ._transports.asgi import ASGIDispatch, ASGITransport
+from ._transports.urllib3 import URLLib3Transport
+from ._transports.wsgi import WSGIDispatch, WSGITransport
 
 __all__ = [
     "__description__",
@@ -46,6 +45,7 @@ __all__ = [
     "stream",
     "codes",
     "ASGIDispatch",
+    "ASGITransport",
     "AsyncClient",
     "Auth",
     "BasicAuth",
@@ -56,7 +56,6 @@ __all__ = [
     "Timeout",
     "ConnectTimeout",
     "CookieConflict",
-    "ConnectionClosed",
     "DecodingError",
     "HTTPError",
     "InvalidURL",
@@ -74,13 +73,14 @@ __all__ = [
     "TooManyRedirects",
     "WriteTimeout",
     "URL",
+    "URLLib3Transport",
     "StatusCode",
     "Cookies",
     "Headers",
     "QueryParams",
     "Request",
-    "TimeoutException",
     "Response",
     "DigestAuth",
     "WSGIDispatch",
+    "WSGITransport",
 ]
