@@ -3,51 +3,45 @@ import typing
 from types import TracebackType
 
 import hstspreload
+import httpcore
 
-from ._auth import Auth, AuthTypes, BasicAuth, FunctionAuth
+from ._auth import Auth, BasicAuth, FunctionAuth
 from ._config import (
     DEFAULT_MAX_REDIRECTS,
     DEFAULT_POOL_LIMITS,
     DEFAULT_TIMEOUT_CONFIG,
     UNSET,
-    CertTypes,
     PoolLimits,
-    ProxiesTypes,
     Proxy,
+    SSLConfig,
     Timeout,
-    TimeoutTypes,
     UnsetType,
-    VerifyTypes,
 )
 from ._content_streams import ContentStream
-from ._dispatch.asgi import ASGIDispatch
-from ._dispatch.base import AsyncDispatcher, SyncDispatcher
-from ._dispatch.connection_pool import ConnectionPool
-from ._dispatch.proxy_http import HTTPProxy
-from ._dispatch.urllib3 import URLLib3Dispatcher
-from ._dispatch.wsgi import WSGIDispatch
 from ._exceptions import HTTPError, InvalidURL, RequestBodyUnavailable, TooManyRedirects
-from ._models import (
-    URL,
-    Cookies,
+from ._models import URL, Cookies, Headers, Origin, QueryParams, Request, Response
+from ._status_codes import codes
+from ._transports.asgi import ASGITransport
+from ._transports.wsgi import WSGITransport
+from ._types import (
+    AuthTypes,
+    CertTypes,
     CookieTypes,
-    Headers,
     HeaderTypes,
-    Origin,
-    QueryParams,
+    ProxiesTypes,
     QueryParamTypes,
-    Request,
     RequestData,
     RequestFiles,
-    Response,
+    TimeoutTypes,
     URLTypes,
+    VerifyTypes,
 )
-from ._status_codes import codes
 from ._utils import (
     NetRCInfo,
     get_environment_proxies,
     get_logger,
     should_not_be_proxied,
+    warn_deprecated,
 )
 
 logger = get_logger(__name__)
@@ -96,9 +90,9 @@ class BaseClient:
         elif isinstance(proxies, (str, URL, Proxy)):
             proxy = Proxy(url=proxies) if isinstance(proxies, (str, URL)) else proxies
             return {"all": proxy}
-        elif isinstance(proxies, AsyncDispatcher):  # pragma: nocover
+        elif isinstance(proxies, httpcore.AsyncHTTPTransport):  # pragma: nocover
             raise RuntimeError(
-                "Passing a dispatcher instance to 'proxies=' is no longer "
+                "Passing a transport instance to 'proxies=' is no longer "
                 "supported. Use `httpx.Proxy() instead.`"
             )
         else:
@@ -107,9 +101,9 @@ class BaseClient:
                 if isinstance(value, (str, URL, Proxy)):
                     proxy = Proxy(url=value) if isinstance(value, (str, URL)) else value
                     new_proxies[str(key)] = proxy
-                elif isinstance(value, AsyncDispatcher):  # pragma: nocover
+                elif isinstance(value, httpcore.AsyncHTTPTransport):  # pragma: nocover
                     raise RuntimeError(
-                        "Passing a dispatcher instance to 'proxies=' is "
+                        "Passing a transport instance to 'proxies=' is "
                         "no longer supported. Use `httpx.Proxy() instead.`"
                     )
             return new_proxies
@@ -424,9 +418,10 @@ class Client(BaseClient):
     that should be followed.
     * **base_url** - *(optional)* A URL to use as the base when building
     request URLs.
-    * **dispatch** - *(optional)* A dispatch class to use for sending requests
+    * **transport** - *(optional)* A transport class to use for sending requests
     over the network.
-    * **app** - *(optional)* An ASGI application to send requests to,
+    * **dispatch** - *(optional)* A deprecated alias for transport.
+    * **app** - *(optional)* An WSGI application to send requests to,
     rather than sending actual network requests.
     * **trust_env** - *(optional)* Enables or disables usage of environment
     variables for configuration.
@@ -446,7 +441,8 @@ class Client(BaseClient):
         pool_limits: PoolLimits = DEFAULT_POOL_LIMITS,
         max_redirects: int = DEFAULT_MAX_REDIRECTS,
         base_url: URLTypes = None,
-        dispatch: SyncDispatcher = None,
+        transport: httpcore.SyncHTTPTransport = None,
+        dispatch: httpcore.SyncHTTPTransport = None,
         app: typing.Callable = None,
         trust_env: bool = True,
     ):
@@ -463,16 +459,24 @@ class Client(BaseClient):
 
         proxy_map = self.get_proxy_map(proxies, trust_env)
 
-        self.dispatch = self.init_dispatch(
+        if dispatch is not None:
+            warn_deprecated(
+                "The dispatch argument is deprecated since v0.13 and will be "
+                "removed in a future release, please use 'transport'"
+            )
+            if transport is None:
+                transport = dispatch
+
+        self.transport = self.init_transport(
             verify=verify,
             cert=cert,
             pool_limits=pool_limits,
-            dispatch=dispatch,
+            transport=transport,
             app=app,
             trust_env=trust_env,
         )
-        self.proxies: typing.Dict[str, SyncDispatcher] = {
-            key: self.init_proxy_dispatch(
+        self.proxies: typing.Dict[str, httpcore.SyncHTTPTransport] = {
+            key: self.init_proxy_transport(
                 proxy,
                 verify=verify,
                 cert=cert,
@@ -482,44 +486,55 @@ class Client(BaseClient):
             for key, proxy in proxy_map.items()
         }
 
-    def init_dispatch(
+    def init_transport(
         self,
         verify: VerifyTypes = True,
         cert: CertTypes = None,
         pool_limits: PoolLimits = DEFAULT_POOL_LIMITS,
-        dispatch: SyncDispatcher = None,
+        transport: httpcore.SyncHTTPTransport = None,
         app: typing.Callable = None,
         trust_env: bool = True,
-    ) -> SyncDispatcher:
-        if dispatch is not None:
-            return dispatch
+    ) -> httpcore.SyncHTTPTransport:
+        if transport is not None:
+            return transport
 
         if app is not None:
-            return WSGIDispatch(app=app)
+            return WSGITransport(app=app)
 
-        return URLLib3Dispatcher(
-            verify=verify, cert=cert, pool_limits=pool_limits, trust_env=trust_env,
+        ssl_context = SSLConfig(
+            verify=verify, cert=cert, trust_env=trust_env
+        ).ssl_context
+
+        return httpcore.SyncConnectionPool(
+            ssl_context=ssl_context,
+            max_keepalive=pool_limits.max_keepalive,
+            max_connections=pool_limits.max_connections,
         )
 
-    def init_proxy_dispatch(
+    def init_proxy_transport(
         self,
         proxy: Proxy,
         verify: VerifyTypes = True,
         cert: CertTypes = None,
         pool_limits: PoolLimits = DEFAULT_POOL_LIMITS,
         trust_env: bool = True,
-    ) -> SyncDispatcher:
-        return URLLib3Dispatcher(
-            proxy=proxy,
-            verify=verify,
-            cert=cert,
-            pool_limits=pool_limits,
-            trust_env=trust_env,
+    ) -> httpcore.SyncHTTPTransport:
+        ssl_context = SSLConfig(
+            verify=verify, cert=cert, trust_env=trust_env
+        ).ssl_context
+
+        return httpcore.SyncHTTPProxy(
+            proxy_url=proxy.url.raw,
+            proxy_headers=proxy.headers.raw,
+            proxy_mode=proxy.mode,
+            ssl_context=ssl_context,
+            max_keepalive=pool_limits.max_keepalive,
+            max_connections=pool_limits.max_connections,
         )
 
-    def dispatcher_for_url(self, url: URL) -> SyncDispatcher:
+    def transport_for_url(self, url: URL) -> httpcore.SyncHTTPTransport:
         """
-        Returns the SyncDispatcher instance that should be used for a given URL.
+        Returns the transport instance that should be used for a given URL.
         This will either be the standard connection pool, or a proxy.
         """
         if self.proxies and not should_not_be_proxied(url):
@@ -537,10 +552,10 @@ class Client(BaseClient):
             )
             for proxy_key in proxy_keys:
                 if proxy_key and proxy_key in self.proxies:
-                    dispatcher = self.proxies[proxy_key]
-                    return dispatcher
+                    transport = self.proxies[proxy_key]
+                    return transport
 
-        return self.dispatch
+        return self.transport
 
     def request(
         self,
@@ -667,22 +682,41 @@ class Client(BaseClient):
                 request = next_request
                 history.append(response)
 
-    def send_single_request(self, request: Request, timeout: Timeout,) -> Response:
+    def send_single_request(self, request: Request, timeout: Timeout) -> Response:
         """
         Sends a single request, without handling any redirections.
         """
 
-        dispatcher = self.dispatcher_for_url(request.url)
+        transport = self.transport_for_url(request.url)
 
         try:
-            response = dispatcher.send(request, timeout=timeout)
+            (
+                http_version,
+                status_code,
+                reason_phrase,
+                headers,
+                stream,
+            ) = transport.request(
+                request.method.encode(),
+                request.url.raw,
+                headers=request.headers.raw,
+                stream=request.stream,
+                timeout=timeout.as_dict(),
+            )
         except HTTPError as exc:
             # Add the original request to any HTTPError unless
             # there'a already a request attached in the case of
             # a ProxyError.
-            if exc.request is None:
-                exc.request = request
+            if exc._request is None:
+                exc._request = request
             raise
+        response = Response(
+            status_code,
+            http_version=http_version.decode("ascii"),
+            headers=headers,
+            stream=stream,  # type: ignore
+            request=request,
+        )
 
         self.cookies.extract_cookies(response)
 
@@ -865,7 +899,7 @@ class Client(BaseClient):
         )
 
     def close(self) -> None:
-        self.dispatch.close()
+        self.transport.close()
         for proxy in self.proxies.values():
             proxy.close()
 
@@ -922,13 +956,13 @@ class AsyncClient(BaseClient):
     that should be followed.
     * **base_url** - *(optional)* A URL to use as the base when building
     request URLs.
-    * **dispatch** - *(optional)* A dispatch class to use for sending requests
+    * **transport** - *(optional)* A transport class to use for sending requests
     over the network.
+    * **dispatch** - *(optional)* A deprecated alias for transport.
     * **app** - *(optional)* An ASGI application to send requests to,
     rather than sending actual network requests.
     * **trust_env** - *(optional)* Enables or disables usage of environment
     variables for configuration.
-    * **uds** - *(optional)* A path to a Unix domain socket to connect through.
     """
 
     def __init__(
@@ -946,10 +980,10 @@ class AsyncClient(BaseClient):
         pool_limits: PoolLimits = DEFAULT_POOL_LIMITS,
         max_redirects: int = DEFAULT_MAX_REDIRECTS,
         base_url: URLTypes = None,
-        dispatch: AsyncDispatcher = None,
+        transport: httpcore.AsyncHTTPTransport = None,
+        dispatch: httpcore.AsyncHTTPTransport = None,
         app: typing.Callable = None,
         trust_env: bool = True,
-        uds: str = None,
     ):
         super().__init__(
             auth=auth,
@@ -962,20 +996,27 @@ class AsyncClient(BaseClient):
             trust_env=trust_env,
         )
 
+        if dispatch is not None:
+            warn_deprecated(
+                "The dispatch argument is deprecated since v0.13 and will be "
+                "removed in a future release, please use 'transport'",
+            )
+            if transport is None:
+                transport = dispatch
+
         proxy_map = self.get_proxy_map(proxies, trust_env)
 
-        self.dispatch = self.init_dispatch(
+        self.transport = self.init_transport(
             verify=verify,
             cert=cert,
             http2=http2,
             pool_limits=pool_limits,
-            dispatch=dispatch,
+            transport=transport,
             app=app,
             trust_env=trust_env,
-            uds=uds,
         )
-        self.proxies: typing.Dict[str, AsyncDispatcher] = {
-            key: self.init_proxy_dispatch(
+        self.proxies: typing.Dict[str, httpcore.AsyncHTTPTransport] = {
+            key: self.init_proxy_transport(
                 proxy,
                 verify=verify,
                 cert=cert,
@@ -986,33 +1027,34 @@ class AsyncClient(BaseClient):
             for key, proxy in proxy_map.items()
         }
 
-    def init_dispatch(
+    def init_transport(
         self,
         verify: VerifyTypes = True,
         cert: CertTypes = None,
         http2: bool = False,
         pool_limits: PoolLimits = DEFAULT_POOL_LIMITS,
-        dispatch: AsyncDispatcher = None,
+        transport: httpcore.AsyncHTTPTransport = None,
         app: typing.Callable = None,
         trust_env: bool = True,
-        uds: str = None,
-    ) -> AsyncDispatcher:
-        if dispatch is not None:
-            return dispatch
+    ) -> httpcore.AsyncHTTPTransport:
+        if transport is not None:
+            return transport
 
         if app is not None:
-            return ASGIDispatch(app=app)
+            return ASGITransport(app=app)
 
-        return ConnectionPool(
-            verify=verify,
-            cert=cert,
+        ssl_context = SSLConfig(
+            verify=verify, cert=cert, trust_env=trust_env
+        ).ssl_context
+
+        return httpcore.AsyncConnectionPool(
+            ssl_context=ssl_context,
+            max_keepalive=pool_limits.max_keepalive,
+            max_connections=pool_limits.max_connections,
             http2=http2,
-            pool_limits=pool_limits,
-            trust_env=trust_env,
-            uds=uds,
         )
 
-    def init_proxy_dispatch(
+    def init_proxy_transport(
         self,
         proxy: Proxy,
         verify: VerifyTypes = True,
@@ -1020,21 +1062,24 @@ class AsyncClient(BaseClient):
         http2: bool = False,
         pool_limits: PoolLimits = DEFAULT_POOL_LIMITS,
         trust_env: bool = True,
-    ) -> AsyncDispatcher:
-        return HTTPProxy(
-            proxy_url=proxy.url,
-            proxy_headers=proxy.headers,
+    ) -> httpcore.AsyncHTTPTransport:
+        ssl_context = SSLConfig(
+            verify=verify, cert=cert, trust_env=trust_env
+        ).ssl_context
+
+        return httpcore.AsyncHTTPProxy(
+            proxy_url=proxy.url.raw,
+            proxy_headers=proxy.headers.raw,
             proxy_mode=proxy.mode,
-            verify=verify,
-            cert=cert,
+            ssl_context=ssl_context,
+            max_keepalive=pool_limits.max_keepalive,
+            max_connections=pool_limits.max_connections,
             http2=http2,
-            pool_limits=pool_limits,
-            trust_env=trust_env,
         )
 
-    def dispatcher_for_url(self, url: URL) -> AsyncDispatcher:
+    def transport_for_url(self, url: URL) -> httpcore.AsyncHTTPTransport:
         """
-        Returns the AsyncDispatcher instance that should be used for a given URL.
+        Returns the transport instance that should be used for a given URL.
         This will either be the standard connection pool, or a proxy.
         """
         if self.proxies and not should_not_be_proxied(url):
@@ -1052,10 +1097,10 @@ class AsyncClient(BaseClient):
             )
             for proxy_key in proxy_keys:
                 if proxy_key and proxy_key in self.proxies:
-                    dispatcher = self.proxies[proxy_key]
-                    return dispatcher
+                    transport = self.proxies[proxy_key]
+                    return transport
 
-        return self.dispatch
+        return self.transport
 
     async def request(
         self,
@@ -1190,17 +1235,36 @@ class AsyncClient(BaseClient):
         Sends a single request, without handling any redirections.
         """
 
-        dispatcher = self.dispatcher_for_url(request.url)
+        transport = self.transport_for_url(request.url)
 
         try:
-            response = await dispatcher.send(request, timeout=timeout)
+            (
+                http_version,
+                status_code,
+                reason_phrase,
+                headers,
+                stream,
+            ) = await transport.request(
+                request.method.encode(),
+                request.url.raw,
+                headers=request.headers.raw,
+                stream=request.stream,
+                timeout=timeout.as_dict(),
+            )
         except HTTPError as exc:
             # Add the original request to any HTTPError unless
             # there'a already a request attached in the case of
             # a ProxyError.
-            if exc.request is None:
-                exc.request = request
+            if exc._request is None:
+                exc._request = request
             raise
+        response = Response(
+            status_code,
+            http_version=http_version.decode("ascii"),
+            headers=headers,
+            stream=stream,  # type: ignore
+            request=request,
+        )
 
         self.cookies.extract_cookies(response)
 
@@ -1383,9 +1447,9 @@ class AsyncClient(BaseClient):
         )
 
     async def aclose(self) -> None:
-        await self.dispatch.close()
+        await self.transport.aclose()
         for proxy in self.proxies.values():
-            await proxy.close()
+            await proxy.aclose()
 
     async def __aenter__(self) -> "AsyncClient":
         return self
